/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.mall.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.mall.entity.CategoryInfo;
import org.springblade.modules.mall.vo.CategoryInfoVO;
import org.springblade.modules.mall.service.ICategoryInfoService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 类别表 控制器
 *
 * @author BladeX
 * @since 2023-02-10
 */
@RestController
@AllArgsConstructor
@RequestMapping("mall/categoryinfo")
@Api(value = "类别表", tags = "类别表接口")
public class CategoryInfoController extends BladeController {

	private final ICategoryInfoService categoryInfoService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入categoryInfo")
	public R<CategoryInfo> detail(CategoryInfo categoryInfo) {
		CategoryInfo detail = categoryInfoService.getOne(Condition.getQueryWrapper(categoryInfo));
		return R.data(detail);
	}

	/**
	 * 分页 类别表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入categoryInfo")
	public R<IPage<CategoryInfo>> list(CategoryInfo categoryInfo, Query query) {
		IPage<CategoryInfo> pages = categoryInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(categoryInfo));
		return R.data(pages);
	}

	/**
	 * 自定义分页 类别表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入categoryInfo")
	public R<IPage<CategoryInfoVO>> page(CategoryInfoVO categoryInfo, Query query) {
		IPage<CategoryInfoVO> pages = categoryInfoService.selectCategoryInfoPage(Condition.getPage(query), categoryInfo);
		return R.data(pages);
	}

	/**
	 * 新增 类别表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入categoryInfo")
	public R save(@Valid @RequestBody CategoryInfo categoryInfo) {
		return R.status(categoryInfoService.save(categoryInfo));
	}

	/**
	 * 修改 类别表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入categoryInfo")
	public R update(@Valid @RequestBody CategoryInfo categoryInfo) {
		return R.status(categoryInfoService.updateById(categoryInfo));
	}

	/**
	 * 新增或修改 类别表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入categoryInfo")
	public R submit(@Valid @RequestBody CategoryInfo categoryInfo) {
		return R.status(categoryInfoService.saveOrUpdate(categoryInfo));
	}

	
	/**
	 * 删除 类别表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(categoryInfoService.removeByIds(Func.toLongList(ids)));
	}

	
}
