package org.springblade.modules.api.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.StatusEnum;
import org.springblade.common.enums.YesOrNoEnum;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.modules.custom.entity.*;
import org.springblade.modules.custom.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * Author pangqijun
 * Date 2023/2/13
 * Description
 */
@RestController
@RequestMapping(CommonConstant.API_URL + "/customWine")
@Api(value = "公共接口", tags = "08.定制模块接口")
public class CustomWineController extends BladeController {

    @Autowired
    private IBottleBodyService bottleBodyService;
    @Autowired
    private IBottleCapService bottleCapService;
    @Autowired
    private IBottleLabelService bottleLabelService;
    @Autowired
    private IWineBoxService wineBoxService;
    @Autowired
    private IWineCaseService wineCaseService;
    @Autowired
    private IWineBoxStuffingService wineBoxStuffingService;
    @Autowired
    private IWineCaseStuffingService wineCaseStuffingService;
    @Autowired
    private IBaseWineService baseWineService;


    @GetMapping("/bottleBody/list")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "获取瓶身列表")
    public R<List<BottleBody>> getBottleBodyList(Query query) {
        LambdaQueryWrapper<BottleBody> wrapper = Wrappers.lambdaQuery(BottleBody.class)
                .eq(BottleBody::getStatus, StatusEnum.ENABLE.getValue()).eq(BottleBody::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<BottleBody> page = bottleBodyService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }

    @GetMapping("/bottleCap/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "获取酒盖列表")
    public R<List<BottleCap>> getBottleCapList(Query query) {
        LambdaQueryWrapper<BottleCap> wrapper = Wrappers.lambdaQuery(BottleCap.class)
                .eq(BottleCap::getStatus, StatusEnum.ENABLE.getValue()).eq(BottleCap::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<BottleCap> page = bottleCapService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }

    @GetMapping("/bottleLabel/list")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "获取标贴列表")
    public R<List<BottleLabel>> getBottleLabelList(Query query) {
        LambdaQueryWrapper<BottleLabel> wrapper = Wrappers.lambdaQuery(BottleLabel.class)
                .eq(BottleLabel::getStatus, StatusEnum.ENABLE.getValue()).eq(BottleLabel::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<BottleLabel> page = bottleLabelService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }

    @GetMapping("/wineBox/list")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "获取酒盒列表")
    public R<List<WineBox>> getWineBoxList(Query query) {
        LambdaQueryWrapper<WineBox> wrapper = Wrappers.lambdaQuery(WineBox.class)
                .eq(WineBox::getStatus, StatusEnum.ENABLE.getValue()).eq(WineBox::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<WineBox> page = wineBoxService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }

    @GetMapping("/wineCase/list")
    @ApiOperationSupport(order = 5)
    @ApiOperation(value = "获取酒箱列表")
    public R<List<WineCase>> getWineCaseList(Query query) {
        LambdaQueryWrapper<WineCase> wrapper = Wrappers.lambdaQuery(WineCase.class)
                .eq(WineCase::getStatus, StatusEnum.ENABLE.getValue()).eq(WineCase::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<WineCase> page = wineCaseService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }

    @GetMapping("/wineCaseStuffing/list")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "获取酒箱填充物列表")
    public R<List<WineCaseStuffing>> getWineCaseStuffingList(Query query) {
        LambdaQueryWrapper<WineCaseStuffing> wrapper = Wrappers.lambdaQuery(WineCaseStuffing.class)
                .eq(WineCaseStuffing::getStatus, StatusEnum.ENABLE.getValue()).eq(WineCaseStuffing::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<WineCaseStuffing> page = wineCaseStuffingService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }

    @GetMapping("/wineBoxStuffing/list")
    @ApiOperationSupport(order = 7)
    @ApiOperation(value = "获取酒盒填充物列表")
    public R<List<WineBoxStuffing>> getWineBoxStuffingList(Query query) {
        LambdaQueryWrapper<WineBoxStuffing> wrapper = Wrappers.lambdaQuery(WineBoxStuffing.class)
                .eq(WineBoxStuffing::getStatus, StatusEnum.ENABLE.getValue()).eq(WineBoxStuffing::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<WineBoxStuffing> page = wineBoxStuffingService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }

    @GetMapping("/baseWine/list")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "获取基酒列表")
    public R<List<BaseWine>> getBaseWineList(Query query) {
        LambdaQueryWrapper<BaseWine> wrapper = Wrappers.lambdaQuery(BaseWine.class)
                .eq(BaseWine::getStatus, StatusEnum.ENABLE.getValue()).eq(BaseWine::getIsDelete, YesOrNoEnum.NO.getValue());
        IPage<BaseWine> page = baseWineService.page(Condition.getPage(query), wrapper);
        return data(page.getRecords());
    }
}
