/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.custom.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.custom.entity.BartenderSkill;
import org.springblade.modules.custom.vo.BartenderSkillVO;
import org.springblade.modules.custom.service.IBartenderSkillService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 调酒师表技能表 控制器
 *
 * @author BladeX
 * @since 2023-02-08
 */
@RestController
@AllArgsConstructor
@RequestMapping("custom/bartenderskill")
@Api(value = "调酒师表技能表", tags = "调酒师表技能表接口")
public class BartenderSkillController extends BladeController {

	private final IBartenderSkillService bartenderSkillService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入bartenderSkill")
	public R<BartenderSkill> detail(BartenderSkill bartenderSkill) {
		BartenderSkill detail = bartenderSkillService.getOne(Condition.getQueryWrapper(bartenderSkill));
		return R.data(detail);
	}

	/**
	 * 分页 调酒师表技能表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入bartenderSkill")
	public R<IPage<BartenderSkill>> list(BartenderSkill bartenderSkill, Query query) {
		IPage<BartenderSkill> pages = bartenderSkillService.page(Condition.getPage(query), Condition.getQueryWrapper(bartenderSkill));
		return R.data(pages);
	}

	/**
	 * 自定义分页 调酒师表技能表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入bartenderSkill")
	public R<IPage<BartenderSkillVO>> page(BartenderSkillVO bartenderSkill, Query query) {
		IPage<BartenderSkillVO> pages = bartenderSkillService.selectBartenderSkillPage(Condition.getPage(query), bartenderSkill);
		return R.data(pages);
	}

	/**
	 * 新增 调酒师表技能表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入bartenderSkill")
	public R save(@Valid @RequestBody BartenderSkill bartenderSkill) {
		return R.status(bartenderSkillService.save(bartenderSkill));
	}

	/**
	 * 修改 调酒师表技能表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入bartenderSkill")
	public R update(@Valid @RequestBody BartenderSkill bartenderSkill) {
		return R.status(bartenderSkillService.updateById(bartenderSkill));
	}

	/**
	 * 新增或修改 调酒师表技能表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入bartenderSkill")
	public R submit(@Valid @RequestBody BartenderSkill bartenderSkill) {
		return R.status(bartenderSkillService.saveOrUpdate(bartenderSkill));
	}

	
	/**
	 * 删除 调酒师表技能表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(bartenderSkillService.removeByIds(Func.toLongList(ids)));
	}

	
}
